"""Random account methods that don't fit anywhere else."""
import os.path
import re

import pexpect
import requests

import ocflib.account.validators as validators
import ocflib.constants as constants


def password_matches(username, password):
    """Returns True if the password matches the user account given"""

    validators.validate_username(username)
    validators.validate_password(username, password, strength_check=False)

    if not validators.user_exists(username):
        raise ValueError("User doesn't exist")

    cmd = 'kinit --no-forwardable -l0 {}@OCF.BERKELEY.EDU'.format(username)
    child = pexpect.spawn(cmd, timeout=10)

    child.expect("{}@OCF.BERKELEY.EDU's Password:".format(username))
    child.sendline(password)

    child.expect(pexpect.EOF)
    child.close()

    return child.exitstatus == 0


def extract_username_from_principal(principal):
    """Extract username from principal.

    >>> extract_username("ckuehl@OCF.BERKELEY.EDU")
    'ckuehl'

    >>> extract_username("ckuehl/admin@OCF.BERKELEY.EDU")
    'ckuehl'
    """

    REGEX = '^([a-z]{3,8})(/[a-z]*)?@OCF\\.BERKELEY\\.EDU$'
    match = re.match(REGEX, principal)

    if not match:
        raise ValueError('Invalid username')

    return match.group(1)


def get_vhost_db():
    """Returns lines from the vhost database. Loaded from the filesystem (if
    available), or from the web if not."""
    try:
        with open(constants.VHOST_DB_PATH) as f:
            return list(map(str.strip, f))
    except IOError:
        # fallback to database loaded from web
        return requests.get(constants.VHOST_DB_URL).text.split('\n')


def get_vhosts():
    """Returns a list of virtual hosts in convenient format.

    >>> parse_vhosts()
    {
        'bpreview.berkeley.edu': {
            'username': 'bpr',
            'aliases': ['bpr.berkeley.edu'],
            'docroot': '/',
            'redirect': None  # format is '/ https://some.other.site/'
        }
    }
    """
    def fully_qualify(host):
        """Fully qualifies a hostname (by appending .berkeley.edu) if it's not
        already fully-qualified."""
        return host if '.' in host else host + '.berkeley.edu'

    vhosts = {}

    for line in get_vhost_db():
        if not line or line.startswith('#'):
            continue

        fields = line.split(' ')

        if len(fields) < 5:
            flags = []
        else:
            flags = re.search('^\[(.*)\]$', fields[4]).group(1).split(',')

        username, host, aliases, docroot = fields[:4]

        redirect = None

        if username.endswith('!'):
            username = username[:-1]
            redirect = '/ https://www.ocf.berkeley.edu/~{}/'.format(username)

        if aliases != '-':
            aliases = list(map(fully_qualify, aliases.split(',')))
        else:
            aliases = []

        vhosts[fully_qualify(username if host == '-' else host)] = {
            'username': username,
            'aliases': aliases,
            'docroot': '/' if docroot == '-' else docroot,
            'redirect': redirect,
            'flags': flags
        }

    return vhosts


def has_vhost(user):
    """Returns whether or not a virtual host is already configured for
    the given user."""
    return any(vhost['username'] == user for vhost in get_vhosts().values())


def home_dir(user):
    """Returns the user's home directory path."""
    return '/' + os.path.join('home', user[0], user[:2], user)


def web_dir(user):
    """Returns the user's web directory path."""
    return '/' + os.path.join('services', 'http', 'users', user[0], user)



"""Methods for working with OCF hosts."""
import ldap3

import ocflib.constants as constants
import ocflib.infra.ldap as ldap


def hosts_by_filter(ldap_filter):
    """Return a list of hosts satisfying the LDAP filter."""

    with ldap.ldap_ocf() as c:
        c.search(
            constants.OCF_LDAP_HOSTS,
            ldap_filter,
            attributes=ldap3.ALL_ATTRIBUTES,
        )

        return [entry['attributes'] for entry in c.response]



import mock
import pytest

from ocflib.misc.whoami import current_uid
from ocflib.misc.whoami import current_user
from ocflib.misc.whoami import current_user_email
from ocflib.misc.whoami import current_user_formatted_email
from ocflib.misc.whoami import current_user_fullname


@pytest.yield_fixture
def mock_uid():
    UID = 28460
    with mock.patch('os.getuid', return_value=UID):
        yield UID


@pytest.yield_fixture
def mock_getpwuid():
    passwd = mock.Mock(
        pw_name='ckuehl',
        pw_passwd='hunter2',
        pw_uid=28460,
        pw_gid=1000,
        pw_gecos='Chris Kuehl',
        pw_dir='/home/c/ck/ckuehl',
        pw_shell='/bin/zsh',
    )
    with mock.patch('pwd.getpwuid', return_value=passwd):
        yield passwd


def test_current_uid(mock_uid):
    assert current_uid() == mock_uid


def test_current_user(mock_uid, mock_getpwuid):
    assert current_user() == mock_getpwuid.pw_name


def test_current_user_fullname(mock_uid, mock_getpwuid):
    assert current_user_fullname() == mock_getpwuid.pw_gecos


def test_current_user_email(mock_uid, mock_getpwuid):
    assert current_user_email() == '{username}@ocf.berkeley.edu'.format(
        username=mock_getpwuid.pw_name,
    )


def test_current_user_formatted_email(mock_uid, mock_getpwuid):
    expected = '{name} <{username}@ocf.berkeley.edu>'.format(
        name=mock_getpwuid.pw_gecos,
        username=mock_getpwuid.pw_name,
    )
    assert current_user_formatted_email() == expected



from setuptools import find_packages
from setuptools import setup

with open('.version') as f:
    VERSION = f.readline().strip()

setup(
    name='ocflib',
    version=VERSION,
    author='Open Computing Facility',
    author_email='help@ocf.berkeley.edu',
    description='libraries for account and server management',
    url='https://www.ocf.berkeley.edu',
    packages=find_packages(),
    package_data={
        'ocflib.account': ['rc/*'],
    },
    install_requires=(
        'colorama',
        'cracklib',
        'dnspython3',
        'ldap3',
        'paramiko',
        # pexpect 3.3 (currently the latest) breaks with multiprocessing/celery
        # https://github.com/pexpect/pexpect/issues/86
        'pexpect<3.2.999',
        'pycrypto',
        'pymysql',
        'pysnmp',
        'redis',
        'requests',
        'sqlalchemy',
    )
)
